use std::str::FromStr;

use sqlx::{
    sqlite::{SqliteConnectOptions, SqliteJournalMode},
    SqlitePool,
};
use tokio::runtime::Builder;

// generated by `sqlx migrate build-script`
fn main() {
    // trigger recompilation when a new migration is added
    println!("cargo:rerun-if-changed=migrations");
    let database_url =
        std::env::var("DATABASE_URL").unwrap_or("sqlite://vault_settings.db3".to_string());
    let options = SqliteConnectOptions::from_str(&database_url)
        .expect("Error parsing database URL")
        .create_if_missing(true)
        .journal_mode(SqliteJournalMode::Wal);
    let rt = Builder::new_current_thread()
        .worker_threads(1)
        .enable_time()
        .build()
        .unwrap();
    rt.block_on(async move {
        connect(options).await;
    });
}

async fn connect(options: SqliteConnectOptions) {
    let pool = SqlitePool::connect_with(options)
        .await
        .expect("Error connecting to database");
    migrate(&pool).await;
}

async fn migrate(pool: &SqlitePool) {
    sqlx::migrate!()
        .run(pool)
        .await
        .expect("Error when applying database migrations");
}
