// Don't show GTK 4.10 deprecations.
// We can't replace them without raising the GTK requirement to 4.10.
#![allow(deprecated)]

use adw::prelude::*;

use relm4::{adw, gtk};
use relm4::{
    prelude::{AsyncComponentParts, SimpleAsyncComponent},
    AsyncComponentSender,
};

use crate::AppMsg;

#[derive(Debug)]
pub enum VaultCreationToggleMsg {
    Show,
    Hide,
}

#[derive(Debug, Clone)]
pub struct VaultCreationSetting {
    pub base_url: String,
    pub identity_url: Option<String>,
    pub email: String,
    // client_cert_path: Option<PathBuf>,
    pub set_active: bool,
}

impl VaultCreationSetting {
    pub fn new(
        base_url: String,
        identity_url: Option<String>,
        email: String,
        set_active: bool,
    ) -> Self {
        Self {
            base_url,
            identity_url,
            email,
            set_active,
        }
    }
}

#[derive(Clone)]
pub struct VaultCreationModel {
    visible: bool,
}

#[relm4::component(pub, async)]
impl SimpleAsyncComponent for VaultCreationModel {
    type Init = ();
    type Input = VaultCreationToggleMsg;
    // TODO maybe we will introduce an own type and map it back in the
    // forward section of the constructing entity
    type Output = AppMsg;

    view! {
        #[root]
        dialog = gtk::Dialog {
            #[watch]
            set_visible: model.visible,
            set_modal: true,

            gtk::Box {
                set_orientation: gtk::Orientation::Vertical,

                #[name = "base_url"]
                adw::EntryRow {
                    set_title: "Base URL",
                },
                #[name = "identity_url"]
                adw::EntryRow {
                    set_title: "Identity URL"
                },
                #[name = "email"]
                adw::EntryRow {
                    set_title: "E-Mail / Username"
                },
                #[name = "toggle_active_vault"]
                gtk::CheckButton {
                    set_label: Some("Use this as active Vault"),
                    set_active: true,
                },
                gtk::Button {
                    set_label: "Confirm",
                    connect_clicked[sender, base_url, identity_url, email, toggle_active_vault] => move |_| {
                        sender.output(AppMsg::CreateVault( VaultCreationSetting {
                            base_url: base_url.to_owned().text().to_string(),
                            identity_url: Some(identity_url.to_owned().text().to_string()),
                            email: email.to_owned().text().to_string(),
                            set_active: toggle_active_vault.to_owned().is_active()
                         })).unwrap();
                    }
                }
            }
        }
    }

    async fn init(
        _init: Self::Init,
        root: Self::Root,
        sender: AsyncComponentSender<Self>,
    ) -> AsyncComponentParts<Self> {
        let model = VaultCreationModel { visible: false };
        let widgets = view_output!();
        AsyncComponentParts { model, widgets }
    }

    async fn update(&mut self, msg: Self::Input, _sender: AsyncComponentSender<Self>) {
        match msg {
            VaultCreationToggleMsg::Show => {
                self.visible = true;
            }
            VaultCreationToggleMsg::Hide => self.visible = false,
        }
    }
}
